unit FormMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, Menus, StdCtrls, ImgList, Buttons, Spin;

Const
APPLICATION_TITLE = 'Delphi Newton Demo Engine';
APPLICATION_VER = '1.1';
APPLICATION_BY = 'Stucuk';

type
  TFrmMain = class(TForm)
    MainMenu1: TMainMenu;
    Panel1: TPanel;
    StatusBar1: TStatusBar;
    MainView: TPanel;
    Help1: TMenuItem;
    About1: TMenuItem;
    Options1: TMenuItem;
    Disable3DView1: TMenuItem;
    Label3: TLabel;
    Label21: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    File1: TMenuItem;
    Exit1: TMenuItem;
    ShowNewtonCollision1: TMenuItem;
    N1: TMenuItem;
    InVehicle1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Idle(Sender: TObject; var Done: Boolean);
    Procedure SetCaption(Filename : String);
    Procedure SetIsEditable;
    procedure MainViewResize(Sender: TObject);
    procedure Disable3DView1Click(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MainViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MainViewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MainViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ShowNewtonCollision1Click(Sender: TObject);
    procedure InVehicle1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FrmMain: TFrmMain;
  BMPOJPG : boolean = true;
  CompressionRate : integer = 1;
  MVW : integer = -1;
  MVH : integer = -1;
  Frame360 : integer = 90;
  FrameAdder : single = 4;
  OldYRot : single;
  FrameCount : integer = 0;
  ParticleFile : String;
implementation

uses ThreeDView_Engine,DND_Global,DND_Newton;

{$R *.dfm}


Function FloatToStr2(Float : Extended) : String; // Makes numbers more understanding
begin

Result := FloatToStrF(Float,ffNumber,7,20);

if Copy(Result,length(Result),1) = '0' then
repeat
Result := Copy(Result,1,length(Result)-1);
until Copy(Result,length(Result),1) <> '0';

if Copy(Result,length(Result),1) = '.' then
Result := Result + '0';
end;

procedure TFrmMain.MainViewMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseButton :=0;
  MainView.Cursor := crCross;
end;

procedure TFrmMain.Exit1Click(Sender: TObject);
begin
Close;
end;

procedure TFrmMain.Idle(Sender: TObject; var Done: Boolean);
begin
if (not MVEnabled) then exit;

DrawFrames;

Done := false;
end;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
SetCaption('');
MainViewWidth := MainView.Width;
MainViewHeight := MainView.Height;

Application.OnIdle := Idle;
Init(MainView.Handle);

SetIsEditable;

Form_StatusBar := @StatusBar1;
end;

Procedure TFrmMain.SetCaption(Filename : String);
begin

If Filename <> '' then
Caption := ' ' + APPLICATION_TITLE + ' v'+APPLICATION_VER + ' [' +Extractfilename(Filename) + ']'
else
Caption := ' ' + APPLICATION_TITLE + ' v'+APPLICATION_VER;

end;

Procedure TFrmMain.SetIsEditable;
var
Value : Boolean;
begin
Value := True;

Options1.Visible := Value;

if Value then
MainView.Cursor := crCross
else
MainView.Cursor := crDefault;

end;

procedure TFrmMain.MainViewResize(Sender: TObject);
begin
if oglloaded then
ResizeMainView(MainView.Width,MainView.Height);
end;

procedure TFrmMain.Disable3DView1Click(Sender: TObject);
begin
MVEnabled := not MVEnabled;
Disable3DView1.Checked := not MVEnabled;

if not MVEnabled then // no point refreshing if opengl is on
MainView.Refresh;
end;

procedure TFrmMain.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
keys[Key] := False;
end;

procedure TFrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
keys[Key] := True;
lastKeyPress := GetTickCount;
end;

procedure TFrmMain.MainViewMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
if (not MVEnabled) then exit;

  if Button = mbLeft  then
  begin
    MouseButton :=1;
    Xcoord := X;
    Ycoord := Y;
  end;
  if Button = mbRight then
  begin
    MouseButton :=2;
    Zcoord := Y;
  end;
end;

procedure TFrmMain.MainViewMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if MouseButton = 1 then
  begin
    xRot := Camera.x_angle;
    yRot := Camera.y_angle;
    xRot := xRot + (Y - Ycoord)/3;  // moving up and down = rot around X-axis
    yRot := yRot + (X - Xcoord)/3;
    Camera.y_angle := YRot;
    Camera.x_angle := XRot;
    Xcoord := X;
    Ycoord := Y;
  end;
end;

procedure TFrmMain.ShowNewtonCollision1Click(Sender: TObject);
begin
ShowNewtonCollision := Not ShowNewtonCollision;
ShowNewtonCollision1.Checked := ShowNewtonCollision;
end;

procedure TFrmMain.InVehicle1Click(Sender: TObject);
begin
InVehicle := Not InVehicle;
Camera.MatrixMode := InVehicle;
InVehicle1.Checked := InVehicle;
end;

end.
