// A much nicer one than VXLSE III
unit ThreeDView_Engine;

interface

Uses Windows,SysUtils,Graphics,Math3d,OpenGL15,dialogs,Classes,ComCtrls;

type
PStatusBar = ^TStatusBar;

var
oglloaded : boolean = false;
MVEnabled : boolean = true;

XRotB,YRotB : boolean;
BGColor,FontColor : TVector3f;

FPS : single;
base : GLuint;		                	// Base Display List For The Font Set
dc  : HDC;     // Device Context
FFrequency : int64;
FoldTime : int64;    // last system time

rc : HGLRC;    // Rendering Context
YRot, XRot, XRot2, YRot2     : glFloat;    // Y Rotation
Depth  : glFloat;
Xcoord, Ycoord, Zcoord : Integer;
MouseButton : Integer;
MainViewWidth,MainViewHeight : integer;

Form_StatusBar : PStatusBar;

Const
MaxBodyCount = 50;

Procedure ResizeMainView(Width,Height : integer);
procedure DrawFrames;
Procedure Init(Handle : HWND);

Function TColorToTVector3f(Color : TColor) : TVector3f;
Function TVector3fToTColor(Vector3f : TVector3f) : TColor;

implementation

Uses Forms,Textures,JPEG,DND_Engine,DND_Global,Cam,Timer,DND_Newton;

procedure BuildFont;			                // Build Our Bitmap Font
var font: HFONT;                	                // Windows Font ID
    gmf : array [0..255] of GLYPHMETRICSFLOAT;		// Address Buffer For Font Storage
begin
  base := glGenLists(256);       	                // Storage For 96 Characters

  SelectObject(DC, font);		       	        // Selects The Font We Want


  font := CreateFont(9, 0,0,0, FW_NORMAL, 0, 0, 0, OEM_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY	, FF_DONTCARE + DEFAULT_PITCH, 'Terminal');
  SelectObject(DC, font);
  wglUseFontBitmaps(DC, 0, 127, base);
end;

procedure KillFont;     		                // Delete The Font
begin
  glDeleteLists(base, 256); 		                // Delete All 96 Characters
end;

procedure glPrint(text : pchar);	                // Custom GL "Print" Routine
begin
  if (text = '') then   			        // If There's No Text
          Exit;					        // Do Nothing

  glPushAttrib(GL_LIST_BIT);				// Pushes The Display List Bits
  glListBase(base);					// Sets The Base Character
  glCallLists(length(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text
  glPopAttrib();								// Pops The Display List Bits
end;

Function TColorToTVector3f(Color : TColor) : TVector3f;
begin

Result.X := GetRValue(Color) / 255;
Result.Y := GetGValue(Color) / 255;
Result.Z := GetBValue(Color) / 255;

end;

Function TVector3fToTColor(Vector3f : TVector3f) : TColor;
begin

Result := RGB(trunc(Vector3f.X*255),trunc(Vector3f.Y*255),trunc(Vector3f.Z*255));

end;

Function CleanVCCol(Color : TColor) : TVector3f;
Var
T : TVector3f;
begin

T.X := GetRValue(Color);
T.Y := GetGValue(Color);
T.Z := GetBValue(Color);

If T.X > 255 then
T.X := 255;

If T.X < 0 then
T.X := 0;

If T.Y > 255 then
T.Y := 255;

If T.Y < 0 then
T.Y := 0;

If T.Z > 255 then
T.Z := 255;

If T.Z < 0 then
T.Z := 0;

T.X := T.X / 255;
T.Y := T.Y / 255;
T.Z := T.Z / 255;

Result := T;

end;

Function CleanV3fCol(Color : TVector3f) : TVector3f;
Var
T : TVector3f;
begin

T.X := Color.X;
T.Y := Color.Y;
T.Z := Color.Z;

If T.X > 255 then
T.X := 255;

If T.X < 0 then
T.X := 0;

If T.Y > 255 then
T.Y := 255;

If T.Y < 0 then
T.Y := 0;

If T.Z > 255 then
T.Z := 255;

If T.Z < 0 then
T.Z := 0;

T.X := T.X / 255;
T.Y := T.Y / 255;
T.Z := T.Z / 255;

Result := T;

end;

function GetCorrectColour(Color : integer) : TVector3f;
begin

end;

Function GetVXLColor(Color,Normal : integer) : TVector3f;
begin
end;


function CleanAngle(Angle : single) : single;
begin
Result := Angle;

If result < 0 then
Result := 360 + Result;

If result > 360 then
Result := Result - 360;
end;

procedure glDraw(); // Main Thing, Draws World
var
x : integer;
Scale,Rotation,Perspective,Position : TVector3f;
begin
//  if (not Frm3DPReview.Showing) then exit;
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);    // Clear The Screen And The Depth Buffer

  if (not MVEnabled) then
  begin
 // glClearColor(0, 0, 0, 1.0);
  Exit;
  end
  else
  glClearColor(BGColor.X, BGColor.Y, BGColor.Z, 1.0);

  if XRotB then
  XRot := XRot + XRot2;

  if YRotB then
  YRot := YRot + YRot2;

  XRot := CleanAngle(XRot);
  YRot := CleanAngle(YRot);

  DrawWorld;

  glPopMatrix;

  glDisable(GL_TEXTURE_2D);

  glLoadIdentity;
  glDisable(GL_DEPTH_TEST);
  glMatrixMode(GL_PROJECTION);
  glPushMatrix;
  glLoadIdentity;
  glOrtho(0,MainViewWidth,0,MainViewHeight,-1,1);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix;
  glLoadIdentity;

  glColor3f(FontColor.X,FontColor.Y,FontColor.Z);

  glRasterPos2i(1,MainViewHeight-9);
  glPrint(pchar('FPS: ' + inttostr(trunc(FPS))));

  glMatrixMode(GL_PROJECTION);
  glPopMatrix;
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix;
  glEnable(GL_DEPTH_TEST);
end;


procedure DrawFrames;
var tmp : int64;
    t2 : double;
begin
  if not oglloaded then exit;

  QueryPerformanceCounter(tmp);
  t2 := tmp-FoldTime;
  FPS := t2/FFrequency;
  FoldTime := TMP;
  FPS := 1/FPS;

  glDraw();                         // Draw the scene
  SwapBuffers(DC);                  // Display the scene
end;

Procedure ResizeMainView(Width,Height : integer);
begin
MainViewWidth := Width;
MainViewHeight := Height;

glResizeWnd(MainViewWidth,MainViewHeight);
end;

Procedure Init(Handle : HWND);
var
i : integer;
pfd : TPIXELFORMATDESCRIPTOR;
pf  : Integer;
begin

InitOpenGL; // Don't forget, or first gl-Call will result in an access violation! exit;

  // OpenGL initialisieren
  dc:=GetDC(Handle);
  h_DC := Dc;

  BGColor   := TColorToTVector3f(RGB(40,111,162));
  FontColor := SetVector(1,1,1);

  // PixelFormat
  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
  pfd.iPixelType:=PFD_TYPE_RGBA;      // PFD_TYPE_RGBA or PFD_TYPEINDEX
  pfd.cColorBits:=32;
  pfd.cStencilBits := 8;
  
  pf :=ChoosePixelFormat(dc, @pfd);   // Returns format that most closely matches above pixel format
  SetPixelFormat(dc, pf, @pfd);

  rc :=wglCreateContext(dc);    // Rendering Context = window-glCreateContext
  wglMakeCurrent(dc,rc);        // Make the DC (Form1) the rendering Context

  ActivateRenderingContext(DC, RC);


  glEnable(GL_TEXTURE_2D);                     // Enable Texture Mapping
  glClearColor(BGColor.X, BGColor.Y, BGColor.Z, 1.0);
  glShadeModel(GL_SMOOTH);                 // Enables Smooth Color Shading
  glClearDepth(1.0);                       // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                 // Enable Depth Buffer
  glDepthFunc(GL_LEQUAL);		           // The Type Of Depth Test To Do

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //Realy Nice perspective calculations

  xRot :=0;
  yRot :=0;
  Depth :=-90;

  oglloaded := true;

  SCREEN_WIDTH := MainViewWidth;
  SCREEN_HEIGHT:= MainViewHeight;

  glResizeWnd(MainViewWidth,MainViewHeight);
  UseCameraByMouse := false;
  TestCollision := True;

//  glEnable(GL_DEPTH_TEST);
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_CULL_FACE);

  Camera := TCamera.Create;
  gTimer := THiResTimer.Create;
  SetupNewton;

  Q3Loaded := Q3Level.LoadBSP('maps\RagdollWaterTest.bsp');
  If Q3Loaded then
  begin
  CreateBSPCollisionTree(Q3Level);

  with Q3Level.PlayerPosition do
  begin
          Camera.Init(FOV, SCREEN_WIDTH, SCREEN_HEIGHT, 1, 1, Angle);
          PlayerPosition.X := Position.X;
          PlayerPosition.Y := Position.Y+100;
          PlayerPosition.Z := Position.Z;
          Camera.position := PlayerPosition;
          Camera.ApplyCamera;
          SetPlayerPos(PlayerPosition);
  end;
  Player_Box := Player_Stand;
  end
  else
  begin
  ShowMessage('Map not found: maps\RagdollWaterTest.bsp');
  Application.Terminate;
  end;

  Randomize;
end;


begin

end.
