unit DND_Engine;

interface

uses Math3d;

procedure glResizeWnd(Width, Height : Integer);
Procedure DrawWorld;

Procedure MakeCube(Size : TVector3f);
Procedure EvaluateKeys;

implementation

uses Windows,OpenGL15,DND_Global,Timer,DND_Newton, Cam, DND_Newton_Vehicle, glmatrixhelper;

procedure glResizeWnd(Width, Height : Integer);
begin
SCREEN_WIDTH := Width;
SCREEN_HEIGHT := Height;

  if (Height = 0) then                // prevent divide by zero exception
    Height := 1;
  glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
  glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
  glLoadIdentity();                   // Reset View
  gluPerspective(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
  glLoadIdentity();                   // Reset View
{$IfDef HELIOS_CONSOLE}
  if HeliosLoaded then
  cons.ResetSize;
{$EndIf}
end;

Procedure DrawWorld;
Var
Velocity,Temp : TVector3f;
begin

gtimer.Refresh;
EvaluateKeys;

glLoadIdentity;
Camera.ApplyCamera;

If UseCameraByMouse then // Move the camera when the mouse moves.
Camera.MoveCameraByMouse;

If InVehicle then
begin
If not Camera.MatrixMode then
Camera.MatrixMode := True;
Vehicle.ProcessInput;
Camera.Matrix := GetVehicleMatrix(Vehicle);
Matrix_Inverse(Camera.Matrix);

//Camera.Matrix[3,0]
//Camera.position := GetVehiclePosition(SetVector(50,0,0));
end
else
begin
Temp := PlayerVelocity;
Camera.Update2(gtimer.FrameTime,Velocity); // Updates the cameras movement using the keys pressed and mouse angles
Velocity := ScaleVector(Velocity,50);
PlayerVelocity := Velocity;

If (Temp.X <> PlayerVelocity.X) or (Temp.Y <> PlayerVelocity.Y) or (Temp.Z <> PlayerVelocity.Z) then
SetPlayerVelocity(Velocity);
Camera.position := GetPlayerPosition;
end;

//SetPlayerPos(Camera.position);

glLoadIdentity;
Camera.ApplyCamera;

DrawNewton;

glLoadIdentity;
Camera.ApplyCamera;

glColor3f(1,1,1);

glCullFace(GL_FRONT);
If Q3Loaded then // If a level is loaded render it
Q3Level.RenderLevel(SetVector(0,0,0));
glCullFace(GL_BACK);
//glDisable(GL_CULL_FACE);

glDisable(GL_DEPTH_TEST);
glDisable(GL_CULL_FACE);
glColor3f(1,1,1);
glMatrixMode(GL_PROJECTION);
glPushMatrix;
glLoadIdentity;
glOrtho(0,640,480,0, -1,1);
glMatrixMode(GL_MODELVIEW);
glPushMatrix;
glLoadIdentity;

//
glEnable(GL_BLEND);
glEnable(GL_TEXTURE_2D);
glBlendFunc(GL_ONE, GL_ONE);
glBindTexture(GL_TEXTURE_2D, NewtonTexID);
glBegin(GL_QUADS);
 glTexCoord2f(0,1); glVertex3f(489, 442, 0);
 glTexCoord2f(1,1); glVertex3f(640, 442, 0);
 glTexCoord2f(1,0); glVertex3f(640, 480, 0);
 glTexCoord2f(0,0); glVertex3f(489, 480, 0);
glEnd;
glDisable(GL_BLEND);
//

glMatrixMode(GL_PROJECTION);
glPopMatrix;
glMatrixMode(GL_MODELVIEW);
glPopMatrix;
glEnable(GL_DEPTH_TEST);

end;

Procedure MakeCube(Size : TVector3f);
begin
glBegin(GL_QUADS);					// Start Drawing The Cube

		glColor3f(0.0,1.0,0.0);			// Set The Color To Green
		glVertex3f( Size.X, Size.Y,-Size.Z);			// Top Right Of The Quad (Top)
		glVertex3f(-Size.X, Size.Y,-Size.Z);			// Top Left Of The Quad (Top)
		glVertex3f(-Size.X, Size.Y, Size.Z);			// Bottom Left Of The Quad (Top)
		glVertex3f( Size.X, Size.Y, Size.Z);			// Bottom Right Of The Quad (Top)

		glColor3f(1.0,0.5,0.0);			// Set The Color To Orange
		glVertex3f( Size.X,-Size.Y, Size.Z);			// Top Right Of The Quad (Bottom)
		glVertex3f(-Size.X,-Size.Y, Size.Z);			// Top Left Of The Quad (Bottom)
		glVertex3f(-Size.X,-Size.Y,-Size.Z);			// Bottom Left Of The Quad (Bottom)
		glVertex3f( Size.X,-Size.Y,-Size.Z);			// Bottom Right Of The Quad (Bottom)

		glColor3f(1.0,0.0,0.0);			// Set The Color To Red
		glVertex3f( Size.X, Size.Y, Size.Z);			// Top Right Of The Quad (Front)
		glVertex3f(-Size.X, Size.Y, Size.Z);			// Top Left Of The Quad (Front)
		glVertex3f(-Size.X,-Size.Y, Size.Z);			// Bottom Left Of The Quad (Front)
		glVertex3f( Size.X,-Size.Y, Size.Z);			// Bottom Right Of The Quad (Front)

		glColor3f(1.0,1.0,0.0);			// Set The Color To Yellow
		glVertex3f( Size.X,-Size.Y,-Size.Z);			// Bottom Left Of The Quad (Back)
		glVertex3f(-Size.X,-Size.Y,-Size.Z);			// Bottom Right Of The Quad (Back)
		glVertex3f(-Size.X, Size.Y,-Size.Z);			// Top Right Of The Quad (Back)
		glVertex3f( Size.X, Size.Y,-Size.Z);			// Top Left Of The Quad (Back)

		glColor3f(0.0,0.0,1.0);			// Set The Color To Blue
		glVertex3f(-Size.X, Size.Y, Size.Z);			// Top Right Of The Quad (Left)
		glVertex3f(-Size.X, Size.Y,-Size.Z);			// Top Left Of The Quad (Left)
		glVertex3f(-Size.X,-Size.Y,-Size.Z);			// Bottom Left Of The Quad (Left)
		glVertex3f(-Size.X,-Size.Y, Size.Z);			// Bottom Right Of The Quad (Left)

		glColor3f(1.0,0.0,1.0);			// Set The Color To Violet
		glVertex3f( Size.X, Size.Y,-Size.Z);			// Top Right Of The Quad (Right)
		glVertex3f( Size.X, Size.Y, Size.Z);			// Top Left Of The Quad (Right)
		glVertex3f( Size.X,-Size.Y, Size.Z);			// Bottom Left Of The Quad (Right)
		glVertex3f( Size.X,-Size.Y,-Size.Z);			// Bottom Right Of The Quad (Right)
	glEnd();
end;

Procedure EvaluateKeys;
begin

if not pause then
if not (keys[Ord('X')]) then
if Player_Box.Y <> Player_Stand.Y then//if CanStand(Camera.position,TVector4fToVector3f(Player_Stand)) then
begin
SetPlayerPos(Camera.position);
Camera.position.Y := Camera.position.Y + (Player_Stand.Y-Player_Box.Y) {- (GRAVITY_VALUE)};
Player_Box := Player_Stand;
Camera.max_speed := Walk_Speed;
end;

//if ingame then
if keys[Ord('P')] then begin
    Pause := not Pause;
    keys[Ord('P')] := false;
    //gTimer.Refresh;
End;

if Pause then Exit; // no movement, etc when paused!

if keys[87] then
begin
Camera.moveForward := true;
Camera.moveUp := true;
end;

if keys[83] then
begin
Camera.moveBackward := true;
Camera.moveDown := true;
end;

if keys[65] then
Camera.moveLeft := true;

if keys[68] then
Camera.moveRight := true;

if not Q3Loaded then exit;

//Q3Level ONLY commands below!

    if (keys[Ord('X')]) then
    begin
      Player_Box := Player_Crouch;
      Camera.max_speed := Crouch_Speed;
    end
    else
    if (keys[VK_SHIFT]) {and (not InLadder)} then
      Camera.max_speed := Run_Speed
    else
    if Camera.max_speed = Run_Speed then
    Camera.max_speed := Walk_Speed;

    if (keys[ord('Z')]) and (CanJump) then
    begin
    CanJump := False;
    NewtonPlayerJump;
    end;

end;

end.
