unit DND_Newton;
// =============================================================================
//
//    DND_Newton - DND's Door to newton
//    Newton Game Dynamics  2003-2004 by Julio Jerez
//     http://physicsengine.com
//    Headerport for Delphi by S.Spasov
//     http://www.geocities.com/delphinewton/
//
// =============================================================================
//   Extracts from Sascha Willems's Newton demos can be found below (Marked with SW)
//   Copyright  2003-2004 by Sascha Willems - http://www.delphigl.de
//   visit the Delphi OpenGL Community - http://www.delphigl.com
// =============================================================================
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
//   So only use this file if you fully unterstand that license!!!
// =============================================================================


interface

Uses OpenGL15,Q3BSP,Math3d,NewtonImport;

procedure CreateBSPCollisionTree(const pSceneBSP : TQuake3BSP);
Function CreateConvexHullFromModel(const pSceneBSP : TQuake3BSP; ModelID : Integer) : PNewtonCollision;
Procedure SetupNewton;
Procedure KillNewton;
Procedure DrawNewton;

Procedure SetPlayerPos(Position : TVector3f);
Procedure SetPlayerVelocity(Velocity : TVector3f);

Function GetPlayerPosition : TVector3f;
Function GetVehiclePosition(P : TVector3f) : TVector3f;

Procedure NewtonPlayerJump;

Procedure SpawnRandomObjects(Position : TVector3f);

Procedure CreateNewtonObject(var pSceneBSP : TQuake3BSP; const ModelId : Integer; NewtonObjectId : Integer; Angles : TVector3f);

implementation

Uses DND_Global,GlMatrixHelper,textures,DND_Engine,Timer,glPrimitives,DND_Newton_Vehicle;

//SW
procedure ApplyForceAndTorqueEvent(const Body : PNewtonBody); cdecl;
var
 Mass   : Single;
 Ixx    : Single;
 Iyy    : Single;
 Izz    : Single;
 Force  : TVector3f;
 Torque : TVector3f;
 TmpV   : TVector3f;
begin
NewtonBodyGetMassMatrix(Body, @Mass, @Ixx, @Iyy, @Izz);

Force := V3(0, Mass * -19.8, 0);
//Force := AddVector(Force,PlayerVelocity);
Torque := V3(0, 0, 0);
NewtonBodyAddForce(Body, @force);
NewtonBodyAddTorque(Body, @torque);
end;

// SW - Stucuk: Modifyed From 3DS Collision Tree to BSP Collision Tree
procedure CreateBSPCollisionTree(const pSceneBSP : TQuake3BSP);
var
 TmpFace : array [0..2] of TVector3f;
 TmpFaceNo : Integer;
 TmpM    : TMatrix4f;
 Min     : TVector3f;
 Max     : TVector3f;
 m,f,c     : Integer;
begin
CollTreeBSP := NewtonCreateTreeCollision(NewtonWorld, nil);
// Go through all meshes and add the faces of it to the tree collision

NewtonTreeCollisionBeginBuild(CollTreeBSP);
if pSceneBSP.numOfFaces > 0 then
   for f := pSceneBSP.Models[0].faceIndex to pSceneBSP.Models[0].faceIndex+pSceneBSP.Models[0].numOfFaces-1 do
   If pSceneBSP.Faces[f].FaceType = 1 then
   If pSceneBSP.Faces[f].numMeshVerts > 0 then
    begin

     for m := 0 to pSceneBSP.Faces[f].numMeshVerts-1 do
     inputBuffer[m] := pSceneBSP.Vertices[pSceneBSP.Faces[f].startVertIndex+pSceneBSP.MeshVertices[pSceneBSP.Faces[f].meshVertIndex+m]];

     c := 0;
     // Stucuk: We need to split it into 3 vertex's per collision face, or newton gets a headake
     // Found that out the hard way!
     for m := 0 to pSceneBSP.Faces[f].numMeshVerts-1 do
     begin
     // Newton likes the points the oposite way to which there stored in the bsp so we reverse it here
     TmpFace[c] := inputBuffer[pSceneBSP.Faces[f].numMeshVerts-1-m].Position;
     inc(c);
     if c = 3 then
     begin
     c := 0;
     NewtonTreeCollisionAddFace(CollTreeBSP, 3, @TmpFace[0], SizeOf(TVector3f), 1);
     end;
     end;

    end;

//We optimise it so it should be faster
NewtonTreeCollisionEndBuild(CollTreeBSP, 1);
BodyBSP := NewtonCreateBody(NewtonWorld, CollTreeBSP);
// Set position to 0/0/0
Matrix_SetIdentity(TmpM);
NewtonBodySetMatrix(BodyBSP, @TmpM[0,0]);
// Set material
NewtonBodySetMaterialGroupID(BodyBSP, MatWorld);
// Get AABB and set limits of the newton world
NewtonCollisionCalculateAABB(CollTreeBSP, @TmpM[0,0], @Min, @Max);
NewtonSetWorldSize(NewtonWorld, @Min, @Max);
NewtonReleaseCollision(NewtonWorld, CollTreeBSP);
end;

// Create a Convex Hull for a model (not the world model)
Function CreateConvexHullFromModel(const pSceneBSP : TQuake3BSP; ModelID : Integer) : PNewtonCollision;
var
 TmpFace : array [0..2] of TVector3f;
 TmpFaceNo : Integer;
 TmpM    : TMatrix4f;
 Min     : TVector3f;
 Max     : TVector3f;
 m,f,c     : Integer;
begin
Result := NewtonCreateTreeCollision(NewtonWorld, nil);

c := 0;
if pSceneBSP.numOfFaces > 0 then
   for f := pSceneBSP.Models[ModelID].faceIndex to pSceneBSP.Models[ModelID].faceIndex+pSceneBSP.Models[ModelID].numOfFaces-1 do
   If pSceneBSP.Faces[f].FaceType = 1 then
   If pSceneBSP.Faces[f].numMeshVerts > 0 then
    begin

     for m := 0 to pSceneBSP.Faces[f].numMeshVerts-1 do
     begin
     inputBuffer[c] := pSceneBSP.Vertices[pSceneBSP.Faces[f].startVertIndex+pSceneBSP.MeshVertices[pSceneBSP.Faces[f].meshVertIndex+m]];
     inc(c);
     end;

    end;

Result := NewtonCreateConvexHull(NewtonWorld,c,@inputBuffer[0].Position.x,SizeOf(TBSPVertex),Nil);

end;

// SW
function GenericContactBegin(const Material : PNewtonMaterial; const Body0 : PNewtonBody; const Body1 : PNewtonBody) : Integer; cdecl;
begin
// Find out if this is the player (as it's the only object that has userdata assigned)
UserData := NewtonBodyGetUserData(Body0);
if UserData = nil then
 UserData := NewtonBodyGetUserData(Body1);
Result := 1;
end;

//SW
function GenericContactProcess(const Material : PNewtonMaterial; const Contact : PNewtonContact) : Integer; cdecl;
begin
// If one of the colliders was the player, then let it jump again
if UserData <> nil then
 Boolean(UserData^) := True;
Result := 1;
end;

// Extracts from SW
Procedure SetupNewton;
var
Dir : Array [0..2] Of Single;
begin

// If there is already a world we want to destroy it and then remake it.
If NewtonLoaded then KillNewton;

// Create the newton world
NewtonWorld := NewtonCreate(nil, nil);
NewtonSetMinimumFrameRate(NewtonWorld, 60);

PrimitiveManager := TNewtonPrimitiveManager.Create;

// Generate materials
// Default (BSP scene, although a collision between this material and something of the same material will never happen)
MatWorld  := NewtonMaterialGetDefaultGroupID(NewtonWorld);
NewtonMaterialSetDefaultSoftness(NewtonWorld, MatWorld, MatWorld, 0.05);
NewtonMaterialSetDefaultElasticity(NewtonWorld, MatWorld, MatWorld, 0.4);
NewtonMaterialSetDefaultCollidable(NewtonWorld, MatWorld, MatWorld, 1);
NewtonMaterialSetDefaultFriction(NewtonWorld, MatWorld, MatWorld, 1, 0.5);
NewtonMaterialSetCollisionCallback(NewtonWorld, MatWorld, MatWorld, nil, @GenericContactBegin, @GenericContactProcess, nil);

// Player
MatPlayer := NewtonMaterialCreateGroupID(NewtonWorld);
NewtonMaterialSetDefaultElasticity(NewtonWorld, MatWorld, MatPlayer, 0.3);
NewtonMaterialSetDefaultFriction(NewtonWorld, MatWorld, MatPlayer, 1.1, 0.7);
NewtonMaterialSetCollisionCallback(NewtonWorld, MatWorld, MatPlayer, nil, @GenericContactBegin, @GenericContactProcess, nil);

// Water
MatWater := NewtonMaterialCreateGroupID(NewtonWorld);
NewtonMaterialSetDefaultFriction(NewtonWorld, MatWorld, MatWater, 1, 1);

// Create player
Collider := NewtonCreateBox(NewtonWorld, Player_Box.X, Player_Box.Y, Player_Box.Z, nil);
PlayerBody := NewtonCreateBody(NewtonWorld, Collider);
Matrix_SetIdentity(PlayerMatrix);
Matrix_SetTransform(PlayerMatrix, V3(0,0,0));
NewtonBodySetMatrix(PlayerBody, @PlayerMatrix[0,0]);
NewtonReleaseCollision(NewtonWorld, Collider);
NewtonBodySetForceAndTorqueCallback(PlayerBody, ApplyForceAndTorqueEvent);
NewtonBodySetMaterialGroupID(PlayerBody, MatPlayer);
// Give it a huge mass, so it makes fun to throw them against other crates ;)
NewtonBodySetMassMatrix(PlayerBody, 10, 10, 10, 10);
NewtonWorldUnfreezeBody(NewtonWorld, PlayerBody);
NewtonBodySetAutoFreeze(PlayerBody, 0);
// Set userdata to a bool indicating weather the player can jump or not
NewtonBodySetUserData(PlayerBody, @CanJump);

Dir[0] := 0;
Dir[1] := 1;
Dir[2] := 0;
NewtonConstraintCreateUpVector(NewtonWorld, @Dir, PlayerBody);

//Vehicle

MatVehicle  := NewtonMaterialGetDefaultGroupID(NewtonWorld);
NewtonMaterialSetDefaultElasticity(NewtonWorld, MatWorld, MatVehicle, 0.1);
NewtonMaterialSetDefaultFriction(NewtonWorld, MatWorld, MatVehicle, 2, 2);
NewtonMaterialSetCollisionCallback(NewtonWorld, MatWorld, MatVehicle, nil, @GenericContactBegin, @GenericContactProcess, nil);


// Build our vehicle
Matrix_SetIdentity(TireMatrix);
Matrix_SetTransform(TireMatrix, V3(0, 100, 0));
Vehicle := TNewtonVehicle.Create(TireMatrix);
Vehicle.CreateCar;
NewtonBodySetAutoFreeze(Vehicle.Body, 0);

// Setup Camera matrix
// We need to adjust the camera's matrix angle and position
Camera.Matrix_Angle_X := 0;
Camera.Matrix_Angle_Y := -90;
Camera.Matrix_Position := SetVector(0, -20, -100);
//

LoadTexture('textures\newtonstrip.jpg', NewtonTexID,False,False,False);

//Make Player
// Generate a display list for the player (faster rendering)
PlayerListID := glGenLists(1);
glNewList(PlayerListID, GL_COMPILE);
 MakeCube(SetVector(5,10,5));
glEndList;

NewtonLoaded := True;
end;

Procedure KillNewton;
begin
If not NewtonLoaded then exit; // no point destroying if its not been created
NewtonDestroy(NewtonWorld);
PrimitiveManager.Destroy;
end;

// SW
procedure Debug_ShowGeometryCollision(const Body : PNewtonBody; VertexCount : Int; const FaceArray : PFloat; FaceId : int); cdecl;
var
 i     : Integer;
 v0,v1 : array[0..2] of Single;
 vA    : array of Single;
begin
if VertexCount = 0 then
 exit;
SetLength(vA, VertexCount*3);
Move(FaceArray^, vA[0], VertexCount*3*SizeOf(Single));
v0[0] := vA[(VertexCount-1)*3];
v0[1] := vA[(VertexCount-1)*3+1];
v0[2] := vA[(VertexCount-1)*3+2];
for i := 0 to VertexCount-1 do
 begin
 v1[0] := vA[i*3];
 v1[1] := vA[i*3+1];
 v1[2] := vA[i*3+2];
 glVertex3f(v0[0], v0[1], v0[2]);
 glVertex3f(v1[0], v1[1], v1[2]);
 v0 := v1;
 end;
end;

// SW
procedure Debug_ShowBodyCollision(const Body : PNewtonBody); cdecl;
begin
NewtonBodyForEachPolygonDo(Body, Debug_ShowGeometryCollision);
end;

// SW - Modifyed
Procedure DrawPlayer;
begin

// Render Player
PlayerPosition := GetPlayerPosition;

NewtonBodyGetMatrix(PlayerBody, @PlayerMatrix[0,0]);
glMultMatrixf(@PlayerMatrix);

glBindTexture(GL_TEXTURE_2D,0);
glColor3f(1,1,1);
//MakeCube(SetVector(Player_Box.X/2,Player_Box.Y/2-13.5,Player_Box.Z/2));
MakeCube(SetVector(Player_Box.X/2,Player_Box.X/2,Player_Box.X/2));
end;

// Draws newton things like the newton logo and collision areas
// SW Extracts
Procedure DrawNewton;
begin

NewtonUpdate(NewtonWorld, gTimer.FrameTime);

//Vehicle.ProcessInput;

PrimitiveManager.Render;

glColor3f(0.5,0.5,0.5);
glLoadIdentity;
Camera.ApplyCamera;

If ShowNewtonCollision then
begin
 glBegin(GL_LINES);
  NewtonWorldForEachBodyDo(NewtonWorld, @Debug_ShowBodyCollision);
 glEnd;
end;

DrawPlayer;

glEnable(GL_CULL_FACE);
end;

Procedure SetPlayerPos(Position : TVector3f);
Var
 VZero : TVector3f;
begin
VZero := SetVector(0,0,0);

       NewtonBodySetVelocity(PlayerBody, @VZero);
       NewtonBodySetOmega(PlayerBody, @VZero);
       NewtonBodySetTorque(PlayerBody, @VZero);
       Matrix_SetIdentity(PlayerMatrix);
       with Position do
       Matrix_SetTransform(PlayerMatrix, V3(X,Y,Z));
       NewtonBodySetMatrix(PlayerBody, @PlayerMatrix[0,0]);
end;

Function GetPlayerPosition : TVector3f;
begin

NewtonBodyGetMatrix(PlayerBody, @PlayerMatrix[0,0]);
Result.X := PlayerMatrix[3,0];
Result.Y := PlayerMatrix[3,1];
Result.Z := PlayerMatrix[3,2];

end;

Function GetVehiclePosition(P : TVector3f) : TVector3f;
begin

NewtonBodyGetMatrix(Vehicle.Body, @Vehicle.Matrix[0,0]);

Result.X := PlayerMatrix[3,0];
Result.Y := PlayerMatrix[3,1];
Result.Z := PlayerMatrix[3,2];
Result := GetPosWithMatrix(P,Vehicle.Matrix);
end;

Procedure NewtonPlayerJump;
var
 TmpV  : TVector3f;
begin

    NewtonBodyGetVelocity(PlayerBody, @TmpV);
    TmpV := V3(TmpV.x, TmpV.y+20, TmpV.z);
    NewtonBodySetVelocity(PlayerBody, @TmpV);

end;

Procedure SetPlayerVelocity(Velocity : TVector3f);
begin
NewtonBodySetVelocity(PlayerBody, @Velocity);
end;

//SW
procedure GetBuoyancyPlane(context : Pointer; const globalSpaceMatrix : PFloat; globalSpacePlane : PFloat); cdecl;
var
 GSPlane  : TVector4f;
begin
GSPlane := V4(0, 1, 0, -25);
Move(GSPlane, globalSpacePlane^, SizeOf(TVector4f));
end;

//SW - Stucuk: Added support for multipul water sources
procedure PhysicsApplyGravityForce(const Body : PNewtonBody); cdecl;
var
 mass    : Single;
 Ixx     : Single;
 Iyy     : Single;
 Izz     : Single;
 Force   : TVector3f;
 Gravity : TVector3f;
 Matrix  : TMatrix4f;
Const
X = 0;
Y = 1;
Z = 2;
begin
NewtonBodyGetMassMatrix(Body, @mass, @Ixx, @Iyy, @Izz);
NewtonBodyGetMatrix(Body, @Matrix[0,0]);

// We check (this is quick and dirty, but works for cubic fluid volumes) if the body is immersed in the fluid
if Q3Level.IsInWater(SetVector(Matrix[3,X],Matrix[3,Y],Matrix[3,Z])) then
 begin
 // Body is in fluid, so we add buoyancy forces
 Gravity := V3(0, -0.15, 0);
 NewtonBodyAddBuoyancyForce(Body, 0.1, 0.9, 0.9, @Gravity, GetBuoyancyPlane, nil);
 // When in a fluid, we don't want our body to get autofreezed
 NewtonBodySetAutoFreeze(Body, 0);
 end
else
 begin
 // Body is not in fluid, so just apply normal gravity
 Force := V3(0, Mass * -19.8, 0);
 NewtonBodySetForce(Body, @Force);
 // When outside the fluid, we can use the autofreezing feature of newton
 NewtonBodySetAutoFreeze(Body, 0);
 end;
end;

// SW
procedure PhysicsSetTransform(const Body : PNewtonBody; const Matrix : PFloat); cdecl;
var
 Primitive : TNewtonPrimitive;
begin
if not Assigned(NewtonBodyGetUserData(Body)) then
 exit;
// Get the primitive by the user data assigned with this body
Primitive := TNewtonPrimitive(NewtonBodyGetUserData(Body));
// Now update it's matrix
Move(Matrix^, Primitive.Matrix, SizeOf(TMatrix4f));
end;

// Same as above but customised for the NewtonObject
procedure PhysicsSetTransformNewtonObject(const Body : PNewtonBody; const Matrix : PFloat); cdecl;
var
 NewtonObject : TNewtonObject;
 Matrix2 : TMatrix4f;
begin
if not Assigned(NewtonBodyGetUserData(Body)) then
 exit;
// Get the primitive by the user data assigned with this body
NewtonObject := TNewtonObject(NewtonBodyGetUserData(Body));

// Now update it's matrix
Move(Matrix^, NewtonObject.Matrix, SizeOf(TMatrix4f));

//NewtonObject.Matrix := Matrix2;
end;

//SW - Procedure Renamed
Procedure SpawnRandomObjects(Position : TVector3f);
var
 TmpBody  : PNewtonBody;
 Collider : PNewtonCollision;
 Inertia  : TVector3f;
 TmpV     : TVector3f;
 Mass     : Single;
 TmpSize  : Single;
 TmpSizeB : Single;
 M        : TMatrix4f;
begin
// Spawn some random primitive to interact with
TmpSize  := 3+Random*2;
TmpSizeB := 3+Random*2;
case Random(4) of
 0 : begin
     Collider := NewtonCreateBox(NewtonWorld, TmpSize*2, TmpSize*2, TmpSize*2, nil);
     TmpBody  := NewtonCreateBody(NewtonWorld, Collider);
     NewtonBodySetUserData(TmpBody, PrimitiveManager.SpawnBox(V3(TmpSize*2, TmpSize*2, TmpSize*2), M));
     // Calculate real moment of inertia
     Mass    := 10*TmpSize;
     Inertia := V3(Mass * (TmpSize*2 * TmpSize*2 + TmpSize*2 * TmpSize*2) / 12,
                   Mass * (TmpSize*2 * TmpSize*2 + TmpSize*2 * TmpSize*2) / 12,
                   Mass * (TmpSize*2 * TmpSize*2 + TmpSize*2 * TmpSize*2) / 12);
     NewtonBodySetMassMatrix(TmpBody, Mass, Inertia.X, Inertia.Y, Inertia.Z);
     end;
 1 : begin
     Collider := NewtonCreateSphere(NewtonWorld, TmpSize, TmpSize, TmpSize, nil);
     TmpBody  := NewtonCreateBody(NewtonWorld, Collider);
     NewtonBodySetUserData(TmpBody, PrimitiveManager.SpawnSphere(TmpSize, M));
     // Calculate real moment of inertia
     Mass    := 10*TmpSize;
     Inertia := V3(2/5 * Mass * Sqr(TmpSize), 2/5 * Mass * Sqr(TmpSize), 2/5 * Mass * Sqr(TmpSize));
     NewtonBodySetMassMatrix(TmpBody, Mass, Inertia.X, Inertia.Y, Inertia.Z);
     end;
 2 : begin
     Collider := NewtonCreateCone(NewtonWorld, TmpSize, TmpSizeB*2, nil);
     TmpBody  := NewtonCreateBody(NewtonWorld, Collider);
     NewtonBodySetUserData(TmpBody, PrimitiveManager.SpawnCone(TmpSize, TmpSizeB*2, M));
     // Calculate real moment of inertia
     Mass    := 10*TmpSize;
     Inertia := V3(3/80 * Mass * Sqr(TmpSizeB) + 3/20 * Mass * Sqr(TmpSize),
                   3/80 * Mass * Sqr(TmpSizeB) + 3/20 * Mass * Sqr(TmpSize),
                   3/80 * Mass * Sqr(TmpSizeB) + 3/20 * Mass * Sqr(TmpSize));
     NewtonBodySetMassMatrix(TmpBody, Mass, Inertia.X, Inertia.Y, Inertia.Z);
     end;
 3 : begin
     Collider := NewtonCreateCylinder(NewtonWorld, TmpSize, TmpSizeB, nil);
     TmpBody  := NewtonCreateBody(NewtonWorld, Collider);
     NewtonBodySetUserData(TmpBody, PrimitiveManager.SpawnCylinder(TmpSize, TmpSizeB, M));
     // Calculate real moment of inertia
     Mass    := 10*TmpSize;
     Inertia := V3(1/12 * Mass * Sqr(TmpSizeB) + 1/4 * Mass * Sqr(TmpSize),
                   1/12 * Mass * Sqr(TmpSizeB) + 1/4 * Mass * Sqr(TmpSize),
                   1/12 * Mass * Sqr(TmpSizeB) + 1/4 * Mass * Sqr(TmpSize));
     NewtonBodySetMassMatrix(TmpBody, Mass, Inertia.X, Inertia.Y, Inertia.Z);
     end;
end;
Matrix_SetIdentity(M);
// Set random spawn position
Matrix_SetTransform(M, AddVector(Position,V3(Random(80)-Random(80), 0, Random(80)-Random(80))));
// Some random orientation
Matrix_SetRotation(M, V3(Random(360), Random(360), Random(360)));
// And some random angular velocity
TmpV := V3(Random*2-Random*2, Random*2-Random*2, Random*2-Random*2);
NewtonBodySetOmega(TmpBody, @TmpV);
NewtonBodySetMatrix(TmpBody, @M[0,0]);
// Set callbacks
NewtonBodySetMaterialGroupID(TmpBody, MatWater);
NewtonBodySetForceAndTorqueCallBack(TmpBody, PhysicsApplyGravityForce);
NewtonBodySetTransformCallBack(TmpBody, PhysicsSetTransform);
NewtonReleaseCollision(NewtonWorld, Collider);
end;

Procedure CreateNewtonObject(var pSceneBSP : TQuake3BSP; const ModelId : Integer; NewtonObjectId : Integer; Angles : TVector3f);
var
 Collider : PNewtonCollision;
 Mass : Single;
 Inertia,Size : TVector3f;
 M : TMatrix4f;
begin


     Collider := CreateConvexHullFromModel(pSceneBSP,ModelId);
     pSceneBSP.NewtonObjects[NewtonObjectId].Body  := NewtonCreateBody(NewtonWorld, Collider);

     Size := SubtractVector(pSceneBSP.Models[ModelId].max,pSceneBSP.Models[ModelId].min);
     Size := DivVector2(Size,2);
     // Calculate real moment of inertia
     Mass    := pSceneBSP.NewtonObjects[NewtonObjectId].Mass;
     Inertia := V3(Mass * (Size.X * Size.Y + Size.X * Size.Z) / 12,
                   Mass * (Size.Y * Size.X + Size.Y * Size.Z) / 12,
                   Mass * (Size.Z * Size.X + Size.Z * Size.Y) / 12);
     NewtonBodySetMassMatrix(pSceneBSP.NewtonObjects[NewtonObjectId].Body, Mass, Inertia.X, Inertia.Y, Inertia.Z);


Matrix_SetIdentity(M);
// Set spawn position
Matrix_SetTransform(M, pSceneBSP.NewtonObjects[NewtonObjectId].Origin);
// Some orientation
Matrix_SetRotation(M, Angles);
NewtonBodySetMatrix(pSceneBSP.NewtonObjects[NewtonObjectId].Body, @M[0,0]);
pSceneBSP.NewtonObjects[NewtonObjectId].Matrix := M;
// Set callbacks
NewtonBodySetMaterialGroupID(pSceneBSP.NewtonObjects[NewtonObjectId].Body, MatWater);
NewtonBodySetForceAndTorqueCallBack(pSceneBSP.NewtonObjects[NewtonObjectId].Body, PhysicsApplyGravityForce);
NewtonBodySetTransformCallBack(pSceneBSP.NewtonObjects[NewtonObjectId].Body, PhysicsSetTransformNewtonObject);
NewtonReleaseCollision(NewtonWorld, Collider);

NewtonBodySetUserData(pSceneBSP.NewtonObjects[NewtonObjectId].Body, pSceneBSP.NewtonObjects[NewtonObjectId]);
end;

end.
